@@ -20,6 +20,7 @@
 
 #include "infohash.h"
 #include "log_enable.h"
+#include "node_export.h"
 
 #include <queue>
 
@@ -265,6 +266,9 @@ public:
 
     virtual void setPushNotificationToken(const std::string&) {};
 
+    virtual void setPushNotificationTopic(const std::string&) {};
+    virtual void setPushNotificationPlatform(const std::string&) {};
+
     /**
      * Call linked callback with a push notification
      * @param notification to process
